#ifndef HPP_HIK_LINEAR_SYSTEM_H
#define HPP_HIK_LINEAR_SYSTEM_H

#include "MatrixAbstractLayer/MatrixAbstractLayer.h"
#include "gikTask/jrlLinearConstraint.h"

/**
\brief This Object stores matrices and values that may constitue a linear equality or inequality system
 */
class ChppHikLinearSystem: public CjrlLinearConstraint
{
public:

    /**
    \brief Constructor
    */
    ChppHikLinearSystem(unsigned int inNparam);

    /**
    \brief Concatenate the given linear systems to this one's.
    \param inIndices if specified, this indicates the lines to be extracted from \param inA and \param inb. All lines are extracted otherwise.
    \return false when a given index is equal or superior to number of lines in inA
     */
    bool expand(const matrixNxP& inA, const vectorN& inb, std::vector<unsigned int>* inIndices = 0);

    /**
    \brief Pad columns of ChppHikLinearSystem::jacobian() with value 0.0. The columns to pas are indicated with the mask vecetor whose size should be equal to number od parameters entered upon construction of instance.
    */
    bool disableColumns(const vectorN& inMask);

    /**
    \brief Empty the matrix and the value.
     */
    void clear();
    
    /**
    \brief Direct access to matrix. Use with CAUTION.
     */
    matrixNxP& matrix();
    
    /**
    \brief Direct access to value. Use with CAUTION.
     */
    vectorN& vector();

    /**
    \brief Get linear system's value
     */
    const vectorN& value();
    
    /**
    \brief Get dimension
     */
    unsigned int dimension() const;

    /**
    \brief Get linear system's matrix
     */
    const matrixNxP& jacobian();

protected:
    vectorN attValue;
    matrixNxP attMatrix;
};

#endif

