#include "boost/numeric/ublas/vector_proxy.hpp"
#include "boost/numeric/ublas/matrix_proxy.hpp"
#include "hpp/hik/solver.hh"
#include "hpp/hik/tools.hh"

#define V3_I  MAL_S3_VECTOR_ACCESS

using namespace boost::numeric::ublas;

ChppHikSolver::ChppHikSolver(CjrlDynamicRobot& inRobot):ChppHikSolverBasic(inRobot.numberDof())
{
    attRobot = &inRobot;

    rootJoint(*(attRobot->rootJoint()));
    H0 =  RootJoint->currentTransformation();
    CurFullConfig = attRobot->currentConfiguration();
    attChangeRootPose = true;
    attChangeRootJoint = false;
}

void ChppHikSolver::rootJoint(CjrlJoint& inRootJoint)
{
    if (RootJoint != &inRootJoint)
    {
        RootJoint = &inRootJoint;
        supportJoints = RootJoint->jointsFromRootToThis();
    }
    attChangeRootJoint = (RootJoint!=attRobot->rootJoint());
}

bool ChppHikSolver::setMask(const vectorN& inMask)
{
    if (inMask.size() != attMask.size())
    {
        std::cout << "ChppHikSolver::setMask() incorrect vector size()"<<std::endl;
        return false;
    }

    attMask = inMask;
    attChangeRootPose = false;
    for (unsigned int i=0;i<6;i++)
        if (attMask(i)!=0)
        {
            attChangeRootPose = true;
            break;
        }
    return true;

}


const vectorN& ChppHikSolver::solutionConfiguration()
{
    return CurFullConfig;
}

const std::pair<vector3d,vector3d>& ChppHikSolver::solutionRootConfiguration()
{
    return attRetPair;
}
const matrix4d& ChppHikSolver::solutionRootPose()
{
    return H0;
}

bool ChppHikSolver::computeSolutionConfiguration(const vectorN& inVelocitySolution)
{
    if (inVelocitySolution.size() != attRobot->numberDof())
    {
        std::cout << "ChppHikSolver::computeRobotConfiguration() incorrect vector size()"<<std::endl;
        return false;
    }
    unsigned int i;
    CurFullConfig = attBackupConfig = attRobot->currentConfiguration();
    CurFullConfig.plus_assign(inVelocitySolution);

    if (!attChangeRootJoint && !attChangeRootPose)
    {
        H0 =  RootJoint->currentTransformation();
    }
    else
    {

        if (attChangeRootJoint)
        {
            Hif = RootJoint->currentTransformation();
            //Let everything be expressed in robot root frame
            for ( i =0; i< 6; i++ )
                CurFullConfig ( i ) = 0.0;

            //update joints transformations from robot root to jacobian root
            for ( i=0; i< supportJoints.size(); i++ )
                supportJoints[i]->updateTransformation ( CurFullConfig );

            //Compute robot root transformation change due to chain of joints between robot root and jacobian root
            Hf = RootJoint->currentTransformation();
            MAL_S4x4_INVERSE(Hf,InvHf,double);
            H0 = Hif * InvHf;
        }

        if (attChangeRootPose)
        {
            //Compute robot root transformation change due to jacobian root transformation change
            ChppHikTools::Matrix4dFromVec(inVelocitySolution, HRC);
            if (attChangeRootJoint)
                H0 = HRC * H0;
            else
            {
                H0 = RootJoint->currentTransformation();
                H0 = HRC * H0;
            }
        }

        if (attChangeRootJoint)
            //Recover joints transformations from root to fixed joint in waist frame
            for ( i=0; i< supportJoints.size(); i++ )
                supportJoints[i]->updateTransformation ( attBackupConfig );
    }

    ChppHikTools::splitM4(H0, TmpR, attRetPair.first);
    ChppHikTools::M3toEulerZYX( TmpR, attRetPair.second);

    for(i=0; i<3;i++)
    {
        CurFullConfig(i) = V3_I(attRetPair.first,i);
        CurFullConfig(3+i) = V3_I(attRetPair.second,i);
    }

    return true;
}
